//**************************************************************************************************
//						БИБЛИОТЕКА RT_HW_BASE.h
// *************************************************************************************************
#ifndef   RT_HW_BASE_h
#define   RT_HW_BASE_h
#define   RT_HW_BASE_VER 		315									//--Номер версии библиотеки;
#define   RT_HW_BASE_VER_NAME 	"3.1.5"								//--Строка с версией библиотеки;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						1.Библиотеки привязки к Arduino
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(ARDUINO) && (ARDUINO >= 100) 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
#include <stdint.h>
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						2.Библиотеки для всех архитектур
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#include <Wire.h>												//--Подключение библиотеки i2c;	   	
#include <SPI.h>												//--Подключение библиотеки SPI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						3.Подключение дополнительных файлов 1
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#include "RT_HW_BASE_01_DEFINED_BOARD.hpp"	    	//--Идентификаторы контроллеров;
#include "RT_HW_BASE_02_DEFINED_PARAM.hpp"			//--Перечисления, структуры, общие макросы;
#include "RT_HW_BASE_03_ARCH_FOR_ALL_BOARD.hpp"	    //--Архитектура для всех контроллеров;
#include "RT_HW_BASE_04_01_BOARD_ARCH_AVR.hpp"	    //--Контроллеры архитектуры AVR;
#include "RT_HW_BASE_04_02_BOARD_ARCH_SAM3.hpp"	    //--Контроллеры архитектуры SAM3;
#include "RT_HW_BASE_04_03_BOARD_ARCH_ESP8266.hpp"	//--Контроллеры архитектуры ESP8266;
#include "RT_HW_BASE_04_04_BOARD_ARCH_ESP32.hpp"	//--Контроллеры архитектуры ESP32;
#include "RT_HW_BASE_04_05_BOARD_ARCH_STM32F1.hpp"	//--Контроллеры архитектуры STM32F1;
#include "RT_HW_BASE_04_06_BOARD_ARCH_STM32F4.hpp"	//--Контроллеры архитектуры STM32F4;
#include "RT_HW_BASE_04_07_BOARD_ARCH_STM32F7.hpp"	//--Контроллеры архитектуры STM32F4;
#include "RT_HW_BASE_04_99_BOARD_ARCH_ANON.hpp"		//--Архитектура ANON (последний в перечни);
#include "RT_HW_BASE_05_DEFINED_STRUCT.hpp"			//--Структуры для использования в RT_HW_BASE.h;
//=================================================================================================
//								КЛАСС RT_HW_BASE.h
//=================================================================================================
class RT_HW_BASE{
public:
//-----Данные--------------------------------------------------------------------------------------
	RT_HW_STRUCT_SHED		shed;
	RT_HW_STRUCT_DEVICE 	device;		//--Данные по устройствам;
	RT_HW_STRUCT_CONSOLE	console;		//--Данные консоли;
//-------------------------------------------------------------------------------------------------
uint32_t buff32;															//--Рабочий регистр;
//-----Методы--------------------------------------------------------------------------------------
RT_HW_BASE();																	//--Конструктор
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						1.Функции работы со временем
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint32_t getPastTime(uint32_t &timeBegin, uint32_t timeEnd);					//--Возвращает разницу между двумя временными точками. 
uint32_t getPastMs 	(uint32_t &timeBegin);										//--Возвращает прошедшее время в мс;
uint32_t getPastMcs	(uint32_t &timeBegin);										//--Возвращает прошедшее время в мкс; 
uint8_t	 timeOut	(uint32_t &timeBegin, uint32_t timeEnd, uint16_t timeMax);	//--Контроль тайм-аута;
uint8_t  timeOutMs	(uint32_t &timeBegin, uint16_t timeMax);					//--Контроль тайм-аута мс;
uint8_t  timeOutMcs	(uint32_t &timeBegin, uint16_t timeMax);					//--Контроль тайм-аута мкс;
uint8_t  period		(uint32_t &timeBegin, uint32_t timeEnd, uint32_t period);	//--Генерация события с точным поддержания периода и защитой от пропуска шага.
void 	 sheduler();															//--Диспетчер задач;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						2.Базовые функции работы с консолью
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void printBegin(uint32_t speed=115200, bool permit=1); 							//--Инициализация консоли;
bool printCheck();																//--Контроль готовности консоли;
void printCR   (char suff='~');													//--Перевод строки;
void printSuff (char suff);														//--Вывод суффикса: ='~' нет вывода; ='Y' перевод строки; =другой символ-вывод 'этого символа;
void printChar (char val, char suff='~');										//--Вывод символа char; 
void printBool (bool v,char sign0='-',char sign1='+',char suff='~'); 			//--Вывод bool;
void printBlank(uint8_t n=1,char blank=' ',char suff='~');						//--Вывод n заполнителей blank;
void printInt8 (int8_t  v,  char mode='I', char suff='~');						//--Вывод int8_t  (mode=char->I,U);
void printInt16(int16_t v,  char mode='I', char suff='~');						//--Вывод int16_t (mode=char->I,U);
void printInt32(int32_t v,  char mode='I', char suff='~');						//--Вывод int32_t (mode=char->I,U);
void printFloat(float   v,  char mode='1', char suff='~');						//--Вывод float   (mode=1,2,3,4);	
void printStr  (String &v,                 char suff='~');						//--Вывод строковой переменной по адресу переменной;
void printTxt  (String  v,                 char suff='~');						//--Вывод строковой переменной;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						3.Форматированный вывод на консоль
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void printVal  (char mode, uint8_t sz, char suff,  int32_t val);				//--Форматированный вывод int32_t (форматы сhar->D,C,S,T,I,U,H,B,1,2,3,4);
void print_v8  (char mode, uint8_t sz, char suff, uint8_t qnt, int8_t  v1=0, int8_t  v2=0, int8_t  v3=0, int8_t  v4=0);	//--Форматированный вывод до 4-х int8_t;
void print_v16 (char mode, uint8_t sz, char suff, uint8_t qnt, int16_t v1=0, int16_t v2=0, int16_t v3=0, int16_t v4=0);	//--Форматированный вывод до 4-х int16_t;
void print_v32 (char mode, uint8_t sz, char suff, uint8_t qnt, int32_t v1=0, int32_t v2=0, int32_t v3=0, int32_t v4=0);	//--Форматированный вывод до 4-х int32_t;
void printFloatx4(uint8_t qnt=0, char suff='~', float v1=0,float v2=0,float v3=0,float v4=0);	//--Форматированный вывод до 4-х float c 2-знаками после запятой;
void printTextStr(String str,      uint8_t len=0, char alig='L',char blank=' ',char suff='~');	//--Форматированный вывод строковой переменной;
void printTextPGM(const char *text,uint8_t len=0, char alig='L',char blank=' ',char suff='~');	//--Форматированный вывод строки из PROGMEM; 	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						4.Вывод на консоль параметров
//  	mode='A' ->  вывод всех элементов в формате int8_t 	с форматированием;
//		mode='B' ->  вывод всех элементов в формате uint8_t c форматированием;
//		mode='U' ->  вывод всех элементов в формате uint8_t c форматированием до трех знаков (вывод байтов);
//		mode='C' ->  вывод элементов 0-127, вместо остальных элементов выводятся заполнители;
//      mode='D' ->  вывод элементов 0-127, остальные элементы не выводятся;
//  	mode='S' ->  вывод всех элементов в формате int8_t без форматирования;
//      mode='P' ->  вывод в формате пинов(для ESP8266,STM32 в текстовой форме),вместо отсутствующих пины(=-1) выводится заполнитель;
//      mode='N' ->  ничего не выводится;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void printNameBoard();							//--Вывод наименования архитектуры и контроллера, номера библиотеки;
void printParamBoard();							//--Вывод параметров контроллера;
void printDepthBoard();							//--Вывод разрядности контроллера;
void printParamWire();							//--Вывод параметров i2c;
void printParamSPI();							//--Вывод параметров SPI;
void printParamUART();							//--Вывод параметров UART (UHS,USS);
void printParamCAN();							//--Вывод параметров CAN;
void printParamBTH();							//--Вывод параметров встроенного BlueTooth;
void printParamETH();							//--Вывод параметров Ethernet;
void printParamWFI();							//--Вывод параметров Wi-Fi;
void printSysParam	(char mode='B');			//--Вывод массива системных параметров;
void printSysPins	(char mode='P');			//--Вывод массива системных пинов;	
void printLinkPins 	(char mode='P');			//--Вывод массива пинов интерфейсов;	
void printNetParam  (char mode='U');			//--Вывод массива пинов сетевых интерфейсов;
void printPinsBoard (char mode='P');			//--Вывод массива пинов контроллера;	
void printPinsTest  (char mode='P');			//--Вывод массива тестовых пинов;
void printAdrTest	(char mode='A');			//--Вывод массива тестовых адресов;
void printTablePinsSTM32();					    //--Вывод таблицы с номерами пинов для STM32;
void printNamePins();							//--Вывод наименование пинов (ESP8266,STM32, по другим ничего не выводится);
//void printScanI2C(uint8_t bus=0);				//--Вывод результатов сканирования шины i2c;
//--------------------------------------------------Вспомогательные функции---------------------------
void printPin(int8_t pin, char suff='~');		//--Вывод номера пина с учетом архитектуры контроллера;
bool checkPinx4(uint8_t qnt, int8_t v1, int8_t v2, int8_t v3, int8_t v4); //--Контроль на наличие до 4-х пинов;
void printPinx4(uint8_t qnt, int8_t v1, int8_t v2, int8_t v3, int8_t v4); //--Вывод до 4-х пинов;
void printArrPGM  (uint8_t kind, char mode); 	//--Вывод номеров пинов типа kind в формате mode
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						5.Вспомогательные  функции работы с консолью
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool checkChange(uint16_t* buff,uint16_t v1,uint16_t v2,uint16_t v3,uint16_t v4);//--Проверка изменения 4-х значений;
char    boolToChar (bool val, char sign_0='-',char sign_1='+');					//--Конвертация bool в char; 
uint8_t getLenBegin(uint8_t len, uint8_t src, char alig);						//--Кол-во заполнителей спереди; 
uint8_t getLenEnd  (uint8_t len, uint8_t src, char alig);						//--Кол-во заполнителей сзади;
uint8_t getLenVal  (char mode, int32_t val);									//--
uint8_t getLenPGM  (const char *text, uint8_t lenMax);							//--Получение длины строки в PROGMEM;
uint8_t getLenStr  (String &str,      uint8_t lenMax);							//--Получение длины строки в оперативной памяти;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// 						6.Вспомогательные  функции разные
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t  getFistBitUint16(uint16_t &reg);										//--Возвращает первый свободный бит регистра reg;
uint16_t maxBitDepth(uint8_t  v);  												//--Возвращает макс.число числа разрядности v;
uint16_t normADC  	(uint16_t v);												//--Нормализация прочитанного ADC значения;
uint16_t normPWM  	(uint16_t v);												//--Нормализация выводимого   PWM значения;
uint16_t normDAC  	(uint16_t v);												//--Нормализация выводимого   DAC значения;
uint16_t makeDelta  (uint16_t v1, uint16_t v2, uint8_t delta);					//--Получения значения при выходе за гистерезис
uint8_t  getNumberVal(uint32_t val, char mode, uint8_t len, uint8_t n);			//--Получение цифры из числа (D,H,B);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//						7.Служебные функции работы с пинами
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint8_t  readArrPGM   (uint8_t kind, uint8_t n); //--Чтение       из массива PGM значения с индексом n;
uint8_t  getLenFullPGM(uint8_t kind);			 //--Получение длины массива PGM;		
uint8_t  getLenPosPGM (uint8_t kind);			 //--Получение длины массива PGM только для значений в диапазоне 0-127;
bool     checkPinPGM  (uint8_t kind, uint8_t n); //--Проверяет наличия пина в массиве PGM;
int8_t   getIntPGM	  (uint8_t kind, uint8_t n); //--Возвращает значение int8_t  из массива PGM по индексу. Если n  >значения массива,то возвращается -1;
uint8_t  getUintPGM   (uint8_t kind, uint8_t n); //--Возвращает значение uint8_t  из массива PGM по индексу. Если n  >значения массива,то возвращается 0;
uint16_t getValPGM	  (uint8_t kind, uint8_t n); //--Возвращает значение int16_t из массива PGM по индексу. Если n+1>значения массива,то возвращается -1;
String	 getNameKindPin(uint8_t kind);			 //--Возвращает символьное обозначение типа пинов.
String 	 getNamePinSTM32(int8_t);
int8_t	 checkPinSTM32  (char reg, uint8_t num); //--Возвращает номер пина по обозначению регистра ('A'-'G') и биту в регистре(0-15);
int8_t	 getPinFromString(String str,uint8_t num);//--Возвращает номер пина из строки, разделенной запятыми; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Работа с пинами чрез прямой доступ к регистрам ввода/вывода
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void 	 directPinSetID	(RT_HW_PIN_DIR_ID &id, int8_t pin); 					//--Настройка параметров пина - адрес регистра,маска,режим использования;
void 	 directPinDigitalWrite(RT_HW_PIN_DIR_ID &id, int8_t pin, uint8_t val);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Работа с пинами средствами Wiring
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
void 	 setPinID       (RT_HW_PIN_DIR_ID &id, int8_t pin, uint8_t kind, char mode='S');	//--Инициализация пина для быстрой работы;
bool 	 pinDigitalRead (RT_HW_PIN_DIR_ID &id);					//--Быстрое дискретное чтение пина;
uint16_t pinAnalogRead  (RT_HW_PIN_DIR_ID &id, uint16_t &ago, char mode='F');		//--Аналоговое чтение пина;	
uint16_t pinTouchRead	(RT_HW_PIN_DIR_ID &id, 				  char mode='F');
void     pinDigitalWrite(RT_HW_PIN_DIR_ID &id, uint8_t val);	//--Быстрая дискретная запись пина;
int16_t  pinReadHALL();
uint16_t pinWritePWM	(RT_HW_PIN_DIR_ID &id, uint16_t val, uint16_t freq=1000, char mode='N');
uint16_t pinWriteDAC	(RT_HW_PIN_DIR_ID &id, uint16_t val);				//--Запись DAC;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Базовые функции работы с UART (n: =0-6 -аппаратные UART; =7-программный UART  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void 	uartBegin    (uint8_t n, uint32_t speed);		//--Инициализация UARTn;
uint8_t	uartSerial   (uint8_t n);						//--Ожидание готовности  UARTn;
uint8_t uartAvailable(uint8_t n);						//--Получение кол-ва байт в буфере чтения UARTn;
uint8_t uartRead     (uint8_t n);						//--Чтение байта         из буфера чтения UARTn;
void 	uartWrite    (uint8_t n, uint8_t val);			//--Запись байта в буфер записи UARTn;
void 	uartPrint    (uint8_t n, String str);			//--Вывод строки в буфер записи UARTn;
bool	checkNumUART (uint8_t num);						//--Проверка наличия в контроллере UARTn; 
void    printNumUART();									//--Вывод на консоль пинов UART в контроллере; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		LINK: РАБОТА С ИНТЕРФЕЙСАМИ
//clockFrequency: the value (in Hertz) of desired communication clock. 
//		Accepted values are 100000 (standard mode) and 400000 (fast //mode). 
//		Some processors also support 10000 (low speed mode), 1000000 (fast mode plus) and 3400000 (high speed mode). 
//		Please refer to //the specific processor documentation to make sure the desired mode is supported.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint32_t convertModeFreqWire(uint8_t mode);	//--Конвертация кода скорости в значение скорости i2c;
void    wireBegin(int8_t sda=-1, int8_t scl=-1, uint8_t slaveAdr=0, uint8_t bus=0); //--Инициализация библиотеки и подключение к i2c;
void    wireBeginTransmission(int8_t adr, 		uint8_t bus=0);	//--Инициирование команды отправки по указанному адресу;
uint8_t wireEndTransmission(					uint8_t bus=0);	//--Завершение процесса передачи, инциированный функцией
void    wireSetClock(uint8_t speed, 			uint8_t bus=0);	//--Установка частоты i2c;

void    wireWrite(uint8_t val,            		uint8_t bus=0);	//--Передача байта;
void    wireWrite(String str,   				uint8_t bus=0);	//--Передача последовательности байт;
void    wireWrite(uint8_t data, uint8_t length, uint8_t bus=0);	//--Передача байтов из массива data в кол-ве length;
void	wireRequestFrom(int8_t adr,uint8_t qnt, uint8_t bus=0);	
bool    findDeviceWire(RT_HW_STRUCT_FIND_I2C &id, uint8_t adr, uint8_t bus=0);	//--Поиск устройства с заданным адресом на шине i2c;
void    scanDeviceWire(uint8_t bus=0);							//--Сканирование шины i2c с номеров bus;
//-------------------------------------------------------------------------------------------------
private:
};  //---END 
//-------------------------------------------------------------------------------------------------
//=================================================================================================
extern RT_HW_BASE 	RT_HW_Base; //--Создание объекта RT_HW_base;
//=================================================================================================
#endif //---RT_HW_BASE_h

