//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Макросы основные для RT_HW_BASE
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_MICROS				micros()					//--Макрос измерения времени в mcs;
#define RT_HW_MILLIS				millis()					//--Макрос измерения времени в ms;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Макросы для работы с PROGMEM
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-----Контроллеры архитектуры ESP8266
#if defined(RT_HW_ARCH_ESP8266)
#define RT_HW_PROGMEM   1
#include <avr/pgmspace.h>			//--Библиотека для работы с PROGMEM;
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(arr+n)		//--Чтение из массива arr байта с номером n
//-----Контроллеры архитектуры ESP32
#elif defined(RT_HW_ARCH_ESP32)
#define RT_HW_PROGMEM   2
#include <pgmspace.h>											//--Работа с PROGMEM;
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(arr+n)		//--Чтение из массива arr байта с номером n
//-----Контроллер MEGA2560(2561)-------------------------------------------------------------------
#elif defined(RT_HW_BOARD_MEGA2560)	//--PROGMEM для MEGA2560;
#define RT_HW_PROGMEM   3
#include <avr/pgmspace.h>			//--Библиотека для работы с PROGMEM;
#ifdef PROGMEM
#undef PROGMEM
#endif
#define PROGMEM __attribute__(( section(".progmem.data") )) 
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(&arr[n])		//--Чтение из массива arr байта с номером n(для <64 kB);
//----Остальные контроллеры-------------------------------------------------------------------------
#else 								//--PROGMEM для остальных контроллеров;
#define RT_HW_PROGMEM  4
#include <avr/pgmspace.h>			//--Библиотека для работы с PROGMEM;	
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(&arr[n])		//--Чтение из массива arr байта с номером n(для <64 kB);
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						Макросы корректировки прерываний для архитектур ESP8266
// Для совместимостью с ESP8266 перед каждой функцией прерываний следует включить RT_HW_ICACHE_RAM_ATTR
// Пример: void RT_HW_ICACHE_RAM_ATTR handleInterrupt(){}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_ARCH_ESP8266)
#define RT_HW_ICACHE_RAM_ATTR		ICACHE_RAM_ATTR				
#else
#define RT_HW_ICACHE_RAM_ATTR		
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						Макросы корректировки прерываний для архитектуры ESP32
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_ARCH_ESP32)
#ifdef  interrupts
#undef  interrupts
#endif
#ifdef  noInterrupts
#undef  noInterrupts
#endif
#define noInterrupts() {portMUX_TYPE mux = portMUX_INITIALIZER_UNLOCKED; portENTER_CRITICAL(&mux)
#define interrupts() portEXIT_CRITICAL(&mux);}
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Макросы для работы вывода пинов на консоль
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-----Контроллеры архитектуры ESP8266
#if  defined(RT_HW_ARCH_ESP8266)
#define RT_HW_PIN_TXT_LEN 3
#define RT_HW_PIN_QNT_LINE 18
//-----Контроллеры архитектуры ESP32
#elif defined(RT_HW_ARCH_ESP32)
#define RT_HW_PIN_TXT_LEN 2
#define RT_HW_PIN_QNT_LINE 20
//-----Контроллеры архитектуры STM32
#elif defined(RT_HW_ARCH_STM32F1) || defined(RT_HW_ARCH_STM32F4) || defined(RT_HW_ARCH_STM32H7) 
#define RT_HW_PIN_TXT_LEN 4
#define RT_HW_PIN_QNT_LINE 16
//-----Другие контроллеры
#else
#define RT_HW_PIN_TXT_LEN 2
#define RT_HW_PIN_QNT_LINE 24
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Макросы вспомогательные
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_CODE_CHAR_BREAK	'~'	//--Код "Запрещающего" символа;
#define RT_HW_CODE_CHAR_BLANK	's'	//--Символ заполнения='пробел' (для удобства применения в блоках FLProg);
//#define RT_HW_ICACHE_RAM_ATTR		//--Атрибут для функций по прерываниям(API) (для совместимости с ESP8266);						 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//enum{RT_HW_PIN_KIND_BAN=14, RT_HW_PIN_INIT_MODE=15, RT_HW_PGM_END=-2};	
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Индексы массивов PGM
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
enum{RT_HW_PGM_SYS_PAR_ID,  RT_HW_PGM_SYS_ADC_ID,	RT_HW_PGM_SYS_TCH_ID,	RT_HW_PGM_SYS_PWM_ID,	RT_HW_PGM_SYS_DAC_ID,
//-------------------------------------------------------------------------------------------------	 
	 RT_HW_PGM_SYS_PIN_ID,
	 RT_HW_PGM_PIN_I2C_ID,  RT_HW_PGM_PIN_SPI_ID,	RT_HW_PGM_PIN_UHS_ID,	RT_HW_PGM_PIN_USS_ID,
	 RT_HW_PGM_PIN_CAN_ID,	RT_HW_PGM_PIN_BTH_ID,	RT_HW_PGM_PIN_ETH_ID,	RT_HW_PGM_PIN_WFI_ID,
//-------------------------------------------------------------------------------------------------
     RT_HW_PGM_PIN_ALL_ID,	RT_HW_PGM_PIN_DIN_ID,	RT_HW_PGM_PIN_ADC_ID,	RT_HW_PGM_PIN_TCH_ID,
	 RT_HW_PGM_PIN_DOT_ID,	RT_HW_PGM_PIN_PWM_ID,	RT_HW_PGM_PIN_DAC_ID,	RT_HW_PGM_PIN_INT_ID, 
	 RT_HW_PGM_PIN_N5V_ID,
//-------------------------------------------------------------------------------------------------
	 RT_HW_PGM_PIN_DBG_ID,	RT_HW_PGM_PIN_OUT_ID,	RT_HW_PGM_PIN_INP_ID, 
	 RT_HW_PGM_PIN_DCT_ID,	RT_HW_PGM_PIN_DEV_ID,	RT_HW_PGM_PIN_CSS_ID,	
	 RT_HW_PGM_PIN_LC6_ID,	RT_HW_PGM_PIN_SSP_ID,   RT_HW_PGM_ADR_I2C_ID,
//-------------------------------------------------------------------------------------------------	 
	 RT_HW_PGM_QNT_ID,		RT_HW_PGM_END=-2
	 //RT_HW_PGM_END=254
	 
	 }; 	  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Идентификаторы скоростей UART (аппаратный и программный);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
enum {RT_HW_UART_SPEED_HS=115200,   RT_HW_UART_SPEED_SS=9600};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Структура для преобразования числа в символы
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_BASE_VAL_ID {uint32_t val; uint16_t len:4,index:4,num:4,mess:4; uint8_t mode:2,minus:1,point:2,pnt:1;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
