#include "RT_HW_BASE.h"	
//=================================================================================================
/**************************************************************************************************
1. Форматированный вывода int32_t (форматы сhar->D,C,S,T,I,U,H,B,1,2,3,4)
   Функция printVal  (char mode, uint8_t sz, char suff,  int32_t val);
   1.1.mode - формат вывода int32_t (форматы сhar->D,C,S,T,I,U,H,B,1,2,3,4):
	'D' - добавление впереди пробела, если число <9;
	'C'	- преобразование числа в символ;
	'S'	- вывод числа как есть (Serial.print(val);
	'T'	- вывод числа           с добавлением    нулей до размера поля sz;
	'I' - вывод числа со знаком с добавлением пробелов до размера поля sz;
	'U' - вывод числа без знака с добавлением пробелов до размера поля sz;
	'H' - вывод HEX числа с префиксом 0x и добав.нулей до размера поля sz;
	'B' - вывод BIN числа с префиксом b  и добав.нулей до размера поля sz;
	'1' - вывод числа/10    в формате float c 1-м  знаком  после запятой;
	'2' - вывод числа/100   в формате float c 2-мя знаками после запятой;
	'3' - вывод числа/1000  в формате float c 3-мя  знаком после запятой;
	'4' - вывод числа/10000 в формате float c 4-мя знаками после запятой;
   1.2.sz - размер выводимого поля. Применяется в форматах T,I,U,H,B.
	   Если sz=0, то устанавливаются следующие значения:
	   'T'->2, 'I'->6, 'U'->5, 'H'->4, 'B'->8.
   1.3.suff - суффикс в конце вывода переменной. Вывод осуществляется функцией printSuff(char):
       '~' - ничего не выводится;	's' - выводится пробел; 'Y' - выполняется перевод строки;
	   'любой другой символ' - выводится этот символ.
   1.4. val - переменная формата int32.
		При форматах вывода 'U','T','H','B' выполняется 
		внутреннее преобразование входной переменной в uint32_t.

2. Форматированный вывод текста из PROGMEM с поддержкой кодировки UTF-8(в т.ч.русский шрифт).
   Функция printTextPGM(const char *text, uint8_t len, char alig, char blank, char suff);
	2.1.const char *text - указатель на текст определенный в памяти типа PROGMEM;
	    Если первый символ текста `~', то вывод заголовка не осуществляется.
	2.2.len - длина выводимого поля. 
		Выводимый текст обрезается сзади, или дополняется заполнителями до длины len.
		Если len=0, текст выводится как есть.
	2.3.alig - выравнивание текста в выводимом поле:
		'L' - влево, 'C' - влево, 'R' - вправо.
	2.4.blank - символ заполнителя поля;
	2.4.suff - суффикс в конце вывода текста. Вывод осуществляется функцией printSuff(char):
       '~' - ничего не выводится;	's' - выводится пробел; 'Y' - выполняется перевод строки;
	   'любой другой символ' - выводится этот символ.   	
***************************************************************************************************/
//=================================================================================================
//   Строковые значения в формате PROGMEM для использования в тесте
//=================================================================================================
const char pinGroup_1[] PROGMEM = "control,debug1,debug2,debug3"; 
const char pinGroup_2[] PROGMEM = "led,button,PWM,ADC"; 
const char pinGroup_3[] PROGMEM = "DAC,TCH"; 
//=================================================================================================
//		1.1 Инициализация консоли
//=================================================================================================
void RT_HW_BASE:: printBegin(uint32_t speed, bool permit) {
if(console.dir==3) {return;}
if(console.dir==0) {if(permit){Serial.begin(speed);console.dir++; console.cnt=0;}} //--Инициализация;
if(console.dir==1) {if(++console.cnt>10) {console.cnt=0; uartSerial(0); if(bitRead(device.uart.ready,0)) {console.ready=console.head=1; console.dir=2;}}} //--Периодическая проверка соединения с UART 
if(console.dir==2) {if(console.resetHead || (++console.cnt>10)) {console.head=0; console.mess=1; console.dir=3;}};
}; 		//--Ожидание сброса или 10 циклов loop(); 		
//=================================================================================================
//		2. Простые функциии вывода на консоль
//=================================================================================================
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		2.1. Контроль готовности консоли
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool RT_HW_BASE:: printCheck() { if(!console.ready) {if(Serial){console.ready=1;}} return console.ready;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		2.2. Перевод строки
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: printCR(char suff){ if(printCheck())  {printSuff(suff); Serial.write('\n');}};				
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		2.3. Вывод суффикса
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: printSuff(char suff){if(printCheck()){if(suff=='~'){return;} 
				  if(suff=='Y'){Serial.write('\n'); return;}; if(suff=='s'){suff=' ';} Serial.write(suff);}}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		2.4. Вывод символа char
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++				  
void RT_HW_BASE:: printChar(char v, char suff) {if(printCheck()){ Serial.write(v); printSuff(suff);}};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		2.5. Вывод bool
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: printBool(bool v,char sign0,char sign1,char suff){if(printCheck()){
	if(v) {Serial.write(sign1);} else {Serial.write(sign0);} printSuff(suff);}};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		2.6. Вывод n заменителей	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: printBlank(uint8_t n,char blank,char suff) {if(printCheck()){
	if(blank==RT_HW_CODE_CHAR_BLANK) {blank=' ';} 
	for(int i=0; i<n; i++){Serial.write(blank);} printSuff(suff);}}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		2.7. Вывод числа int
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: printInt8 (int8_t  v, char mode, char suff) {if(printCheck()){if(mode=='I'){Serial.print(v);} else {Serial.print( (uint8_t)v);} printSuff(suff);}};
void RT_HW_BASE:: printInt16(int16_t v, char mode, char suff) {if(printCheck()){if(mode=='I'){Serial.print(v);} else {Serial.print((uint16_t)v);} printSuff(suff);}};
void RT_HW_BASE:: printInt32(int32_t v, char mode, char suff) {if(printCheck()){if(mode=='I'){Serial.print(v);} else {Serial.print((uint32_t)v);} printSuff(suff);}};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		2.8. Вывод числа float
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: printFloat(float v, char mode, char suff) {if(printCheck()){
if(mode=='0') {Serial.print(v,0);} 
if(mode=='1') {Serial.print(v,1);}
if(mode=='2') {Serial.print(v,2);}
if(mode=='3') {Serial.print(v,3);}	
if(mode=='4') {Serial.print(v,4);}	
printSuff(suff);}};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		2.8. Вывод строковой переменной
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: printStr (String &v,char suff){if(printCheck()){Serial.print(v); printSuff(suff);}};
void RT_HW_BASE:: printTxt (String  v,char suff){if(printCheck()){Serial.print(v); printSuff(suff);}};
//=================================================================================================
//		3. Форматированный вывод
//=================================================================================================
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		3.1 Форматированный вывод int32_t (форматы сhar->D,C,S,T,I,U,H,B,1,2,3,4) 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: printVal(char mode, uint8_t sz, char suff, int32_t val){if(printCheck()){ uint8_t len; String str;
//------Вывод с добавление впереди пробела до 2-х знаков-------------------------------------------
if(mode=='D'){if((val>=0)&&(val<10)){Serial.write(' ');} Serial.print(val);}
//------Вывод одного символа-----------------------------------------------------------------------
if(mode=='C') {Serial.write(char(val&0xFF));} 	
//------Простой вывод числа------------------------------------------------------------------------
if(mode=='S') {Serial.print(val);}
//------Вывод в формате времени последних (до sz значащих знаков) с добавлением ' ' до длины sz----
if(mode=='T') {	if(sz==0){sz=2;} if(sz>4){sz=4;} str=String((uint16_t)val); len=str.length(); 
				if(len>sz){str=str.substring(len-sz,len); len=sz;}	printBlank((sz-len),'0');  Serial.print(str);}																		
//------Вывод в INT-формате последних (до sz значащих знаков) с добавлением ' ' до длины sz-------
if(mode=='I') {	if(sz==0){sz=6;} if(sz>10){sz=10;} str=String( (int32_t)val); len=str.length(); 
				if(len>sz){str=str.substring(len-sz,len); len=sz;}	printBlank((sz-len),' ');  Serial.print(str);} 
 //------Вывод в UINT-формате последних (до sz значащих знаков) с добавлением ' ' до длины sz-------
if(mode=='U') {	if(sz==0){sz=5;} if(sz>10){sz=10;} str=String((uint32_t)val); len=str.length(); 
				if(len>sz){str=str.substring(len-sz,len); len=sz;}	printBlank((sz-len),' ');  Serial.print(str);} 
 //------Вывод в HEX-формате последних (до sz значащих знаков) с добавлением '0' до длины sz-------
if(mode=='H') {	if(sz==0){sz=4;} if(sz> 8){sz= 8;} str=String((uint32_t)val,HEX); str.toUpperCase(); len=str.length(); 
				if(len>sz){str=str.substring(len-sz,len); len=sz;}	printChar('0','x'); printBlank((sz-len),'0');  Serial.print(str);}
//------Вывод в BIN-формате. Вывод первых знаков('S'=32, 's'=16, 'B'=8, 'b'=4)---------------------
if(mode=='B') {	if(sz==0){sz=4;} if(sz>32){sz=32;} str=String((uint32_t)val,BIN); str.toUpperCase(); len=str.length(); 
				if(len>sz) {str=str.substring(len-sz,len); len=sz;}	printChar('b'); printBlank((sz-len),'0');  Serial.print(str);}
//------Вывод в формате с точкой-------------------------------------------------------------------
if(mode=='1') {Serial.print(float(val)/10,1);} 									
if(mode=='2') {Serial.print(float(val)/100,2);}
if(mode=='3') {Serial.print(float(val)/1000,3);} 									
if(mode=='4') {Serial.print(float(val)/10000,4);}					
printSuff(suff);}};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  	3.2. Форматированный вывод до 4-х int8_t.
//	Если suff='C' (continue), в конце добавляется разделитель запятая, перевод строки не осуществляется.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void  RT_HW_BASE:: print_v8 (char mode, uint8_t sz, char suff, uint8_t qnt, int8_t v1, int8_t v2, int8_t v3, int8_t v4){if(printCheck()){
if(qnt>0){                printVal(mode,sz,'~',v1);} 	if(qnt>1){printChar(','); printVal(mode,sz,'~',v2);} 
if(qnt>2){printChar(','); printVal(mode,sz,'~',v3);} 	if(qnt>3){printChar(','); printVal(mode,sz,'~',v4);} 	
if(suff=='C') {printChar(','); return;}
printChar(';',suff);}};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  	3.3. Форматированный вывод до 4-х int16_t
//	Если suff='C' (continue), в конце добавляется разделитель запятая, перевод строки не осуществляется.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void  RT_HW_BASE:: print_v16(char mode, uint8_t sz, char suff, uint8_t qnt, int16_t v1, int16_t v2, int16_t v3, int16_t v4){if(printCheck()){
if(qnt>0){                printVal(mode,sz,'~',v1);} 	if(qnt>1){printChar(','); printVal(mode,sz,'~',v2);} 
if(qnt>2){printChar(','); printVal(mode,sz,'~',v3);} 	if(qnt>3){printChar(','); printVal(mode,sz,'~',v4);} 	
if(suff=='C') {printChar(','); return;}
printChar(';',suff);}};		
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  	3.4. Форматированный вывод до 4-х int32_t
//	Если suff='C' (continue), в конце добавляется разделитель запятая, перевод строки не осуществляется.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void  RT_HW_BASE:: print_v32(char mode, uint8_t sz, char suff, uint8_t qnt, int32_t v1, int32_t v2, int32_t v3, int32_t v4){if(printCheck()){
if(qnt>0){                printVal(mode,sz,'~',v1);} 	if(qnt>1){printChar(','); printVal(mode,sz,'~',v2);} 
if(qnt>2){printChar(','); printVal(mode,sz,'~',v3);} 	if(qnt>3){printChar(','); printVal(mode,sz,'~',v4);} 	
if(suff=='C') {printChar(','); return;}
printChar(';',suff);}};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		3.5. Форматированный вывод до 4-х float	c 2-знаками после запятой
//	Если suff='C' (continue), в конце добавляется разделитель запятая, перевод строки не осуществляется.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void  RT_HW_BASE:: printFloatx4(uint8_t qnt, char suff, float v1, float v2, float v3, float v4){if(printCheck()){
//bool suffCR; if(qnt>10) {qnt=qnt-10; suffCR=0;} else {suffCR=1;}
if(qnt>0){ 					printFloat(v1,2);}	
if(qnt>1){printChar(','); 	printFloat(v2,2);} 
if(qnt>2){printChar(','); 	printFloat(v3,2);} 
if(qnt>3){printChar(','); 	printFloat(v4,2);} 
if(suff=='C') {printChar(','); return;}
printChar(';',suff);}};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		3.6. Форматированный вывод строковой переменной
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: printTextStr(String str, uint8_t len, char alig, char blank, char suff) { if(printCheck()) {uint8_t curLen=0;  
if(len>0) {str.substring(0,len);} curLen= getLenStr(str, len);               
printBlank(getLenBegin(len,curLen,alig),blank); printStr(str); printBlank(getLenEnd(len,curLen,alig),blank); printSuff(suff); }};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		3.7. Форматированный вывод текста из PROGMEM
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: printTextPGM(const char *text, uint8_t len, char alig, char blank, char suff) { 
if(printCheck()) {uint8_t curLen,j; char buff;                 
curLen=getLenPGM(text,len); j=0;
printBlank(getLenBegin(len,curLen,alig),blank); 
for(uint8_t i=0; i<curLen; i++){buff=pgm_read_byte(text+j); Serial.print(buff); j++; 
				                if((uint8_t)buff>=0xC0){buff=pgm_read_byte(text+j); Serial.print(buff); j++;}}
printBlank(getLenEnd(len,curLen,alig),blank); 
printSuff(suff);}};	
//=================================================================================================
//		4. ВСПОМОГАТЕЛЬНЫЕ ФУНКЦИИ
//=================================================================================================
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.1. Конвертация bool в char------------------------------------------------------------ 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
char    RT_HW_BASE::boolToChar (bool val,char sign_0,char sign_1){if(val) return sign_1; else return sign_0;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.2. Получение кол-ва заполнителей спереди (для форматированного вывода)----------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE::getLenBegin(uint8_t len,uint8_t src,char alig){if(len<=src) return 0;
		switch(alig){default:return 0; case 'C':return (len-src-(len-src)/2); case 'R':return (len-src);}};  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.3. Получение кол-ва заполнителей сзади (для форматированного вывода)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE::getLenEnd  (uint8_t len,uint8_t src,char alig){if(len<=src) return 0;
switch(alig){default:return 0; case 'C':return (len-src-getLenBegin(len,src,alig)); case 'L':return (len-src);}}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++		
//		4.4 Получение длины текста String (utf-8, блокирующий символ RT_HW_CODE_CHAR_BREAK '~')
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE:: getLenStr(String &str, uint8_t lenMax){char buff; uint8_t LEN=0; 
const char * data=str.c_str();	
for(uint8_t i=0; i<=160; i++){buff=data[i]; 
if((buff=='\0') || (buff==RT_HW_CODE_CHAR_BREAK) || (LEN>=lenMax)) {break;}
if(((uint8_t)buff)<0xC0) {LEN++;}}		   
return LEN;};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++												   
//		4.5.Получение длины текста PROGMEM (utf-8, блокирующий символ RT_HW_CODE_CHAR_BREAK '~')
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE:: getLenPGM(const char *text, uint8_t lenMax){uint8_t buff,j,LEN; if(lenMax==0) {lenMax=80;} j=LEN=0;
while (1) {buff=pgm_read_byte(text+j);	j++;
		   if((buff==0)   || (buff==RT_HW_CODE_CHAR_BREAK) || (LEN>=lenMax) || (j>160)) {break;}
		   if( buff>=0xC0){j++; buff=pgm_read_byte(text+j);} LEN++;} return LEN;}	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.6.Получение кол-ва знаков в числах  с оcнованиями 10,16,2 (DEC(I,U),HEX,BIN)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE:: getLenVal(char mode, int32_t val){
uint32_t v, buff; uint8_t len=0; uint8_t bs;
if(mode=='I') {if(val<0) {v=-val;} else {v=val;}	
								buff=bs=10;  len=1; for(int8_t i=0;i<10;i++) {if(v<buff){break;} buff=buff*bs; len++;} if(val<0) {len++;}} 
if(mode=='U') {v=(uint32_t)val; buff=bs=10;  len=1; for(int8_t i=0;i<10;i++) {if(v<buff){break;} buff=buff*bs; len++;}} 
if(mode=='H') {v=(uint32_t)val;	buff=bs=0xF; len=1; for(int8_t i=0;i<10;i++) {if(v<buff){break;} buff=buff*bs; len++;}} 
if(mode=='B') {v=(uint32_t)val;	buff=bs=0x1; len=1; for(int8_t i=0;i<10;i++) {if(v<buff){break;} buff=buff*bs; len++;}} 
return len;};						
/**************************************************************************************************
						ТЕКСТ ДЛЯ БЛОКОВ FLProg
Форматированный вывода int16_t c форматированным заголовком.(
1.Перед переменной выводится заголовок с форматом, определяемые параметрами пользователя:
	1.1.ЗАГОЛОВОК - текст UTF-8(в т.ч.русский шрифт), который размещается в памяти PROGMEM.
	    Если первый символ текста `~', то вывод заголовка не осуществляется.
	1.2.ДЛИНА_ЗАГОЛОВКА - длина выводимого поля вместе текстом ЗАГОЛОВОК. 
		Выводимый текст обрезается сзади, или дополняется сзади пробелами до этой длины.
		Если ДЛИНА_ЗАГОЛОВКА=0, текст ЗАГОЛОВКА выводится как есть.
	1.3.СУФФИКС_ЗАГОЛОВКА - действие после вывода поля с ЗАГОЛОВКОМа:
		'~'-ничего не выводится; 's'-выводится пробел; 'Y'-выполняется перевод строки;
	  	'любой другой символ'-выводится этот символ.
2.После заголовка выводится переменная  форматом, определяемые параметрами пользователя:
	2.1.ФОРМАТ_ПЕРЕМЕННОЙ - (D,C,S,T,I,U,H,B,1,2,3,4):
	'D' - добавление впереди пробела, если число <9;
	'C'	- преобразование числа в символ;
	'S'	- вывод числа как есть (Serial.print(val);
	'T'	- вывод числа           с добавлением    нулей до размера поля sz;
	'I' - вывод числа со знаком с добавлением пробелов до размера поля sz;
	'U' - вывод числа без знака с добавлением пробелов до размера поля sz;
	'H' - вывод HEX числа с префиксом 0x и добав.нулей до размера поля sz;
	'B' - вывод BIN числа с префиксом b  и добав.нулей до размера поля sz;
	'1' - вывод числа/10    в формате float c 1-м  знаком  после запятой;
	'2' - вывод числа/100   в формате float c 2-мя знаками после запятой;
	'3' - вывод числа/1000  в формате float c 3-мя  знаком после запятой;
	'4' - вывод числа/10000 в формате float c 4-мя знаками после запятой;
   2.2.РАЗМЕР_ПОЛЯ_ПЕРЕМЕННОЙ - размер выводимого поля вместе с переменной.
	   Применяется в форматах T,I,U,H,B. Если РАЗМЕР_ПОЛЯ_ПЕРЕМЕННОЙ =0, 
	   то устанавливаются следующие значения: 'T'->2, 'I'->6, 'U'->5, 'H'->4, 'B'->8.
   2.3.СУФФИКС_ПЕРЕМЕННОЙ - суффикс в конце вывода переменной. 
       '~'-ничего не выводится; 's'-выводится пробел; 'Y'-выполняется перевод строки;
	   'любой другой символ' - выводится этот символ.
   2.4. val - переменная формата int32.
		При форматах вывода 'U','T','H','B' выполняется 
		внутреннее преобразование входной переменной в uint32_t.
3.Разрешение вывода регулируется флагами консоли, параметром пользователя, внешним разрешением EN.
	3.1.флаг head - устанавливается консолью после инициализации UART.
		Это особенно важно для контроллеров DUE,STM32 без аппаратного чипа UART-USB.
	    Флаг head сбрасывается через программную установку флага reset, 
		или самостоятельно через 10 циклов loop().
	3.2.флаг mess - уставливается после установки и сброса флага head.
	3.3.параметр пользователя УСЛОВИЕ ВЫВОДА:
		'C'-вывод осуществляется по флагу mess при первом вхождении в блок, 
			  далее по изменению значения переменной.
		'P'-вывод осуществляется по флагу mess при каждом вхождении в блок.
		'H'-вывод осуществляется по флагу head.		
	3.4.вывод осуществляется только при разрешение EN=1.
	    Вход EN можно использовать для циклического вывода при УСЛОВИЕ ВЫВОДА='P', для чего
		на вход EN периодически подается "Событие" (HIGH в течении одного цикла loop()). 
---------------------------------------------------------------------------------------------------
ecoins@mail.ru 19.11.2020	   	
***************************************************************************************************/
